from dataclasses import dataclass, field

from typing import List

from proto_proto_pscr.cb_psc_event_pb2 import (
    CbAlert,
    CbThreatIndicator,
    ThreatScore,
    CbKeyValue,
    CbWatchlistHit,
    CbAlertStateDetails,
    CbAlertNote,
    CbAlertUpdateDetails,
    CbLegacyV6AlertDetails,
    CbIDSMatch,
    CbMicroIDSMetadata,
    CbAlertClassificationPredictionList,
    CbAlertAggregation,
    CbMDRAlertClassificationPredictionList,
    CbNtaMetadata,
)

from sensor_comms.protobuf_requests.utilities.utilities import create_default_uuid, create_default_sha256_hash

from sensor_comms.protobuf_requests.event_messages import MessageDataClass
from sensor_comms.protobuf_requests.event_messages.threats.threat_indicator import ThreatIndicator
from sensor_comms.protobuf_requests.event_messages.threats.threat_score import ThreatScoreDC
from sensor_comms.protobuf_requests.event_messages.key_value import KeyValue
from sensor_comms.protobuf_requests.event_messages.alert.alert_aggregation import AlertAggregation
from sensor_comms.protobuf_requests.event_messages.alert.alert_state_details import AlertStateDetails
from sensor_comms.protobuf_requests.event_messages.alert.alert_note import AlertNote
from sensor_comms.protobuf_requests.event_messages.alert.alert_update_details import AlertUpdateDetails
from sensor_comms.protobuf_requests.event_messages.alert.legacy_v6_alert_details import LegacyV6AlertDetails
from sensor_comms.protobuf_requests.event_messages.alert.mdr_alert_classification_prediction_list import (
    MDRAlertClassificationPredictionList,
)
from sensor_comms.protobuf_requests.event_messages.alert.alert_classification_prediction_list import (
    AlertClassificationPredictionList,
)
from sensor_comms.protobuf_requests.event_messages.alert.watchlist_hit import WatchlistHit
from sensor_comms.protobuf_requests.event_messages.alert.ids_match import IDSMatch
from sensor_comms.protobuf_requests.event_messages.alert.micro_ids_metadata import MicroIDSMetadata
from sensor_comms.protobuf_requests.event_messages.alert.nta_metadata import NtaMetadata


@dataclass
class Alert(MessageDataClass):
    # pylint: disable=too-many-instance-attributes
    # Fields map to CbAlert fields
    """
    Dataclass that maps to the CbAlert protobuf message
    """
    alert_id_guid: str = field(default_factory=create_default_uuid)
    alert_hash: str = field(default_factory=create_default_sha256_hash)
    alert_confidence: int = field(default=CbAlert.CbAlertConfidence.Value('ALERT_CONFIDENCE_UNKNOWN'))
    alert_execution_state: int = field(default=CbAlert.ThreatExecutionState.Value('THREAT_EXECUTION_UNKNOWN'))
    alert_description: str = field(default=None)
    alert_producer: str = field(default=None)
    threat_indicators: List[CbThreatIndicator] = field(
        default_factory=list, metadata={'dataclass_type': ThreatIndicator}
    )
    alert_creation_timestamp: str = field(default=None)
    alert_threat_category: int = field(default=CbAlert.ThreatCategory.Value('UNKNOWN'))
    alert_threat_score: ThreatScore = field(default=None, metadata={'dataclass_type': ThreatScoreDC})
    alert_type: int = field(default=CbAlert.AlertType.Value('ALERT_TYPE_UNKNOWN'))
    watchlist_hit: CbWatchlistHit = field(default=None, metadata={'dataclass_type': WatchlistHit})
    ids_match: CbIDSMatch = field(default=None, metadata={'dataclass_type': IDSMatch})
    microids_metadata: CbMicroIDSMetadata = field(default=None, metadata={'dataclass_type': MicroIDSMetadata})
    nta_metadata: CbNtaMetadata = field(default=None, metadata={'dataclass_type': NtaMetadata})
    associated_alert_id_guid: str = field(default=None)
    alert_dispatch_tags: List[str] = field(default_factory=list)
    alert_precedence: int = field(default_factory=list)
    alert_update_timestamp: str = field(default=None)
    defense_analytics_primary_event: bool = field(default=False)
    alert_first_seen_timestamp: str = field(default=None)
    extended_props: List[CbKeyValue] = field(default_factory=list, metadata={'dataclass_type': KeyValue})
    deprecated_alert_id_guids: List[str] = field(default_factory=list)
    ttp_chain_id_guid: List[str] = field(default_factory=list)
    mdr_alert: bool = field(default=False)
    user_state_details: CbAlertStateDetails = field(default=None, metadata={'dataclass_type': AlertStateDetails})
    mdr_state_details: CbAlertStateDetails = field(default=None, metadata={'dataclass_type': AlertStateDetails})
    added_alert_note: CbAlertNote = field(default=None, metadata={'dataclass_type': AlertNote})
    alert_update_details: CbAlertUpdateDetails = field(default=None, metadata={'dataclass_type': AlertUpdateDetails})
    highest_alert_execution_state: int = field(default=CbAlert.ThreatExecutionState.Value('THREAT_EXECUTION_UNKNOWN'))
    alert_last_seen_timestamp: str = field(default=None)
    legacy_v6_alert_details: CbLegacyV6AlertDetails = field(
        default=None, metadata={'dataclass_type': LegacyV6AlertDetails}
    )
    alert_classification: CbAlertClassificationPredictionList = field(
        default=None, metadata={'dataclass_type': AlertClassificationPredictionList}
    )
    alert_aggregation: CbAlertAggregation = field(default=None, metadata={'dataclass_type': AlertAggregation})
    mdr_alert_classification: CbMDRAlertClassificationPredictionList = field(
        default=None, metadata={'dataclass_type': MDRAlertClassificationPredictionList}
    )
    alert_tags: List[str] = field(default_factory=list)
    tms_rule_id: str = field(default=None)
    threat_name: str = field(default=None)
    observation_type: int = field(default=CbAlert.ObservationType.Value('OBSERVATION_TYPE_UNKNOWN'))
    backend_timestamp: str = field(default=None)
    backend_update_timestamp: str = field(default=None)
    user_update_timestamp: str = field(default=None)
    alert_notes_present: bool = field(default=False)
    mdr_alert_notes_present: bool = field(default=False)
    rule_id: str = field(default=None)
    alert_metadata: List[CbKeyValue] = field(default_factory=list, metadata={'dataclass_type': KeyValue})
    aggregation_timestamp: str = field(default=None)
    classification_timestamp: str = field(default=None)
    suppression_timestamp: str = field(default=None)
    _message_type: str = field(init=False, repr=False, default='CbAlert')
