from dataclasses import dataclass, field

from proto_proto_pscr.cb_psc_event_pb2 import (
    CbProcessOp,
    CbFileOp,
    CbRegistryOp,
    CbLoadModuleOp,
    CbNetOp,
    CbVolumeOp,
    CbSystemOp,
    CbTelemetryOp,
    CbNoOp,
    CbDeviceOp,
    CbLogonOp,
    CbContainerOp,
    CbEventHeader,
    CbProcessObject,
    CbSystemObject,
)

from sensor_comms.protobuf_requests.event_messages import MessageDataClass

from sensor_comms.protobuf_requests.event_messages.event.event_header import EventHeader
from sensor_comms.protobuf_requests.event_messages.objects.process_object import ProcessObject
from sensor_comms.protobuf_requests.event_messages.operations.network_operation import NetworkOperation
from sensor_comms.protobuf_requests.event_messages.operations.process_operation import ProcessOperation
from sensor_comms.protobuf_requests.event_messages.operations.file_operation import FileOperation
from sensor_comms.protobuf_requests.event_messages.operations.registery_operation import RegistryOperation
from sensor_comms.protobuf_requests.event_messages.operations.load_module_operation import LoadModuleOperation
from sensor_comms.protobuf_requests.event_messages.operations.volume_operation import VolumeOperation
from sensor_comms.protobuf_requests.event_messages.operations.system_operation import SystemOperation
from sensor_comms.protobuf_requests.event_messages.operations.telemetry_operation import TelemetryOperation
from sensor_comms.protobuf_requests.event_messages.operations.no_op_operation import NoOp
from sensor_comms.protobuf_requests.event_messages.operations.device_operation import DeviceOperation
from sensor_comms.protobuf_requests.event_messages.operations.logon_operation import LogonOperation
from sensor_comms.protobuf_requests.event_messages.operations.container_operation import ContainerOperation
from sensor_comms.protobuf_requests.event_messages.objects.system_object import SystemObject


@dataclass
class Event(MessageDataClass):
    """
    Event dataclass. Fields correspond to fields of CbEventMsg
    """

    # pylint: disable=too-many-instance-attributes
    header: CbEventHeader = field(
        default_factory=EventHeader().create_message_object, metadata={'dataclass_type': EventHeader}
    )
    actor: CbProcessObject = field(
        default_factory=ProcessObject().create_message_object, metadata={'dataclass_type': ProcessObject}
    )
    parent_actor: CbProcessObject = field(default=None, metadata={'dataclass_type': ProcessObject})
    grandparent_actor: CbProcessObject = field(default=None, metadata={'dataclass_type': ProcessObject})
    process_op: CbProcessOp = field(default=None, metadata={'dataclass_type': ProcessOperation})
    filemod_op: CbFileOp = field(default=None, metadata={'dataclass_type': FileOperation})
    regmod_op: CbRegistryOp = field(default=None, metadata={'dataclass_type': RegistryOperation})
    modload_op: CbLoadModuleOp = field(default=None, metadata={'dataclass_type': LoadModuleOperation})
    netconn_op: CbNetOp = field(default=None, metadata={'dataclass_type': NetworkOperation})
    volume_op: CbVolumeOp = field(default=None, metadata={'dataclass_type': VolumeOperation})
    system_op: CbSystemOp = field(default=None, metadata={'dataclass_type': SystemOperation})
    telemetry_op: CbTelemetryOp = field(default=None, metadata={'dataclass_type': TelemetryOperation})
    no_op: CbNoOp = field(default=None, metadata={'dataclass_type': NoOp})
    device_op: CbDeviceOp = field(default=None, metadata={'dataclass_type': DeviceOperation})
    logon_op: CbLogonOp = field(default=None, metadata={'dataclass_type': LogonOperation})
    container_op: CbContainerOp = field(default=None, metadata={'dataclass_type': ContainerOperation})
    system: CbSystemObject = field(default=None, metadata={'dataclass_type': SystemObject})
    storage_class: str = field(default=None)
    exec_parent_actor: CbProcessObject = field(default=None, metadata={'dataclass_type': ProcessObject})
    exec_grandparent_actor: CbProcessObject = field(default=None, metadata={'dataclass_type': ProcessObject})
    _message_type: str = field(init=False, repr=False, default='CbEventMsg')
