from typing import List

from dataclasses import dataclass, field

from proto_proto_pscr.cb_psc_event_pb2 import CbEventProtocolVersion, CbRulePolicyGuid, CbAlert, CbEventHeader

from sensor_comms.protobuf_requests.event_messages import MessageDataClass
from sensor_comms.protobuf_requests.event_messages.event.rule_policy_guid import RulePolicyGuid
from sensor_comms.protobuf_requests.event_messages.alert.alert import Alert


@dataclass
class EventHeader(MessageDataClass):
    """
    EventHeader dataclass. Fields correspond to fields of CbEventHeader
    """

    # pylint: disable=too-many-instance-attributes
    # Fields map to CbEventHeader fields
    event_type: int = field(default=CbEventHeader.CbEventType.Value('TYPE_PROCESS'))
    version: int = field(default=CbEventProtocolVersion.Value('PROTOCOL_VERSION_CURRENT'))
    device_timestamp: str = field(default=None)  # Set timestamp fields with GetCurrentTime()
    magic: int = field(default=0xDEADBEEF)
    triggers: List[CbRulePolicyGuid] = field(
        default_factory=RulePolicyGuid().create_message_object, metadata={'dataclass_type': RulePolicyGuid}
    )
    disposition: int = field(default=None)
    operation_tags: List[str] = field(default_factory=list)
    flags: int = field(default=None)
    sensor_action: int = field(default=CbEventHeader.CbSensorAction.Value('ACTION_BLOCK'))
    sensor_version: str = field(default='3.8.0.677')
    event_hash: str = field(default=None)
    rule_check_type: int = field(default=None)
    device_location: int = field(default=None)
    event_id: str = field(default=None)
    triggered_alert: List[CbAlert] = field(default_factory=list, metadata={'dataclass_type': Alert})
    associated_alerts: List[CbAlert] = field(default_factory=list, metadata={'dataclass_type': Alert})
    report_subcode: int = field(default=None)
    cloud_dispatch_tags: List[str] = field(default_factory=list)
    system_boot_time: str = field(default=None)
    sensor_enabled_time: str = field(default=None)
    event_source: int = field(default=None)
    windows_event_id: int = field(default=None)
    _message_type: str = field(init=False, repr=False, default='CbEventHeader')
