from dataclasses import dataclass, field
from typing import List

from proto_proto_pscr.cb_psc_event_pb2 import (
    CbDigitalSignature,
    CbFileObject,
    CbPEFileMetaData,
    CbVolumeObject,
    CbMachOFileMetaData,
    ThreatScore,
    CbStateMachineState,
    CbAlternateDataStreamData,
)

from sensor_comms.protobuf_requests.event_messages import MessageDataClass
from sensor_comms.protobuf_requests.event_messages.files.digital_signature import DigitalSignature
from sensor_comms.protobuf_requests.event_messages.files.pe_file_metadata import PEFileMetaData
from sensor_comms.protobuf_requests.event_messages.threats.threat_score import ThreatScoreDC
from sensor_comms.protobuf_requests.event_messages.objects.state_machine_state import StateMachineState
from sensor_comms.protobuf_requests.event_messages.objects.volume_object import VolumeObject
from sensor_comms.protobuf_requests.event_messages.objects.alternate_data_stream_data import AlternateDataStreamData
from sensor_comms.protobuf_requests.event_messages.files.mach_o_file_metadata import MachOFileMetaData

from sensor_comms.protobuf_requests.utilities.utilities import create_default_md5_hash, create_default_sha256_hash


@dataclass
class FileObject(MessageDataClass):
    # pylint: disable=too-many-instance-attributes
    # Fields map to CbFileObject fields
    """
    Dataclass that maps to the CbFileObject protobuf message
    """
    path: str = field(default='C:\\WINDOWS\\system32\\services.exe')
    md5_hash: str = field(default_factory=create_default_md5_hash)
    sha256_hash: str = field(default_factory=create_default_sha256_hash)
    cloud_reputation: int = field(default=None)
    effective_reputation: int = field(default=CbFileObject.Reputation.Value('REP_LOCAL_WHITE'))
    reputation_source: int = field(default=CbFileObject.EffectiveReputationSource.Value('REP_SRC_IGNORE'))
    digital_signatures: List[CbDigitalSignature] = field(
        default_factory=list, metadata={'dataclass_type': DigitalSignature}
    )
    tags: List[str] = field(default_factory=list)
    architecture: int = field(default=CbFileObject.MachineArchitecture.Value('ARCH_X86'))
    file_size: int = field(default=None)
    flags: int = field(default=None)
    device_type: int = field(default=CbFileObject.CbDeviceType.Value('FILE_DEVICE_TYPE_FIXED'))
    state: List[CbStateMachineState] = field(default_factory=list, metadata={'dataclass_type': StateMachineState})
    file_metadata: CbPEFileMetaData = field(default=None, metadata={'dataclass_type': PEFileMetaData})
    file_key: int = field(default=None)
    yara_rule_tags: List[str] = field(default_factory=list)
    volume_object: CbVolumeObject = field(default=None, metadata={'dataclass_type': VolumeObject})
    macho_file_metadata: CbMachOFileMetaData = field(default=None, metadata={'dataclass_type': MachOFileMetaData})
    total_bytes_hashed: int = field(default=None)
    threat_scores: List[ThreatScore] = field(default=None, metadata={'dataclass_type': ThreatScoreDC})
    alternate_data_streams: CbAlternateDataStreamData = field(
        default=None, metadata={'dataclass_type': AlternateDataStreamData}
    )
    apc_risk_level: int = field(default=None)
    threat_name: str = field(default=None)
    threat_type: str = field(default=None)
    tlsh_hash: str = field(default=None)
    _message_type: str = field(init=False, repr=False, default='CbFileObject')
