from dataclasses import dataclass, field
from typing import List

from proto_proto_pscr.cb_psc_event_pb2 import (
    CbNetConnectionObject,
    CbIpAddress,
    KubernetesNetInfo,
    CbKeyValue,
    CbEndpointInfo,
)

from sensor_comms.protobuf_requests.event_messages import MessageDataClass

from sensor_comms.protobuf_requests.event_messages.network.ip_address import IpAddress
from sensor_comms.protobuf_requests.event_messages.key_value import KeyValue
from sensor_comms.protobuf_requests.event_messages.network.kubernetes.kubernetes_net_info import KubernetesNetInfoDC
from sensor_comms.protobuf_requests.event_messages.network.endpoint_info import EndpointInfo


@dataclass
class NetworkConnectionObject(MessageDataClass):
    # pylint: disable=too-many-instance-attributes
    # Fields map to CbNetworkConnectionObject fields
    """
    Dataclass that maps to the CbNetworkConnectionObject protobuf message
    """
    create_time: str = field(default=None)
    protocol: int = field(default=CbNetConnectionObject.ProtocolType.Value('PROTO_UDP'))
    domain: str = field(default=None)
    inbound: bool = field(default=False)
    proxy_connection: bool = field(default=False)
    proxy_ip_address: CbIpAddress = field(default=None, metadata={'dataclass_type': IpAddress})
    proxy_port: int = field(default=None)
    proxy_domain: str = field(default=None)
    remote_ip_address: CbIpAddress = field(default=None, metadata={'dataclass_type': IpAddress})
    remote_port: int = field(default=None)
    local_ip_address: CbIpAddress = field(default=None, metadata={'dataclass_type': IpAddress})
    local_port: int = field(default=None)
    bytes_transmitted: int = field(default=None)
    bytes_received: int = field(default=None)
    extended_props: List[CbKeyValue] = field(default_factory=list, metadata={'dataclass_type': KeyValue})
    http_request: str = field(default=None)
    http_host: str = field(default=None)
    community_id: str = field(default=None)
    ja3_fingerprint: str = field(default=None)
    tags: List[str] = field(default_factory=list)
    application_protocol_tags: List[str] = field(default_factory=list)
    kubernetes_info: KubernetesNetInfo = field(default=None, metadata={'dataclass_type': KubernetesNetInfoDC})
    remote_endpoint_info: CbEndpointInfo = field(default=None, metadata={'dataclass_type': EndpointInfo})
    network_profile: int = field(default=CbNetConnectionObject.NetworkProfile.Value('NETWORK_PROFILE_UNKNOWN'))
    _message_type: str = field(init=False, repr=False, default='CbNetConnectionObject')
