from dataclasses import dataclass, field
from typing import List

from proto_proto_pscr.cb_psc_event_pb2 import (
    CbFileObject,
    CbStateMachineState,
    CbProcessUniqueId,
    CbWin32ProcessData,
    CbUser,
    CbContainerObject,
)

from sensor_comms.protobuf_requests.event_messages import MessageDataClass

from sensor_comms.protobuf_requests.event_messages.objects.process_unique_id import ProcessUniqueId
from sensor_comms.protobuf_requests.event_messages.objects.state_machine_state import StateMachineState
from sensor_comms.protobuf_requests.event_messages.objects.win32_process_data import Win32ProcessData
from sensor_comms.protobuf_requests.event_messages.objects.file_object import FileObject
from sensor_comms.protobuf_requests.event_messages.objects.user import User
from sensor_comms.protobuf_requests.event_messages.objects.container_object.container_object import ContainerObject

from sensor_comms.protobuf_requests.utilities.utilities import create_default_thread_id


@dataclass
class ProcessObject(MessageDataClass):
    # pylint: disable=too-many-instance-attributes
    # Fields map to CbProcessObject fields
    """
    Dataclass that maps to the CbProcessObject protobuf message
    """
    main_module: CbFileObject = field(
        default_factory=FileObject().create_message_object, metadata={'dataclass_type': FileObject}
    )
    pid: CbProcessUniqueId = field(
        default_factory=ProcessUniqueId().create_message_object, metadata={'dataclass_type': ProcessUniqueId}
    )
    parent_pid: CbProcessUniqueId = field(default=None, metadata={'dataclass_type': ProcessUniqueId})
    user_id: str = field(default='SA-1-2-3-4')
    user_name: str = field(default='CB INTERNAL\\USER_1')
    cmdline: str = field(default='c:\\program files (x86)\\svchost.exe \\qwer sad olasdjf')
    modload_count: int = field(default=0)
    filemod_count: int = field(default=0)
    netconn_count: int = field(default=0)
    regmod_count: int = field(default=0)
    childproc_count: int = field(default=0)
    crossproc_count: int = field(default=0)
    crossproc_target_count: int = field(default=0)
    thread_id: int = field(default_factory=create_default_thread_id)
    tags: List[str] = field(default_factory=list)
    state: List[CbStateMachineState] = field(default_factory=list, metadata={'dataclass_type': StateMachineState})
    win32_process_data: CbWin32ProcessData = field(default=None, metadata={'dataclass_type': Win32ProcessData})
    modload_script_count: int = field(default=0)
    exec_pid: CbProcessUniqueId = field(default=None, metadata={'dataclass_type': ProcessUniqueId})
    loaded_modules: List[CbFileObject] = field(default_factory=list, metadata={'dataclass_type': FileObject})
    unique_logon_id: str = field(default=None)
    user_info: CbUser = field(default=None, metadata={'dataclass_type': User})
    container_info: CbContainerObject = field(default=None, metadata={'dataclass_type': ContainerObject})
    pid_namespace_value: int = field(default=None)
    spoofed_cmdline: str = field(default=None)
    _message_type: str = field(init=False, repr=False, default='CbProcessObject')
